@@ -1,6 +1,6 @@
 /* ncftp.h
  *
- * Copyright (c) 1996-2006 Mike Gleason, NcFTP Software.
+ * Copyright (c) 1996-2008 Mike Gleason, NcFTP Software.
  * All rights reserved.
  *
  */
@@ -13,10 +13,10 @@ extern "C"
 {
 #endif	/* __cplusplus */
 
-#define kLibraryVersion "@(#) LibNcFTP 3.2.0 (August 5, 2006)"
+#define kLibraryVersion "@(#) LibNcFTP 3.2.3 (August 18, 2008)"
 
 /* This is used to verify validty of the data passed in. */
-#define kLibraryMagic "LibNcFTP 3.2.0"
+#define kLibraryMagic "LibNcFTP 3.2.3"
 
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 	/* Includes for Windows */
@@ -83,9 +83,8 @@ extern "C"
 #	ifndef _INC_SETJMP
 #		include <setjmp.h>
 #	endif
-#	define longest_int LONGLONG
-#	define longest_int LONGLONG
-#	define longest_uint ULONGLONG
+#	define longest_int __int64
+#	define longest_uint __uint64
 #	ifndef HAVE_LONG_LONG
 #		define HAVE_LONG_LONG 1
 #	endif
@@ -110,8 +109,12 @@ extern "C"
 #	ifndef main_void_return_t
 #		define main_void_return_t void
 #	endif
-#	define strcasecmp stricmp
-#	define strncasecmp strnicmp
+#	ifndef strcasecmp
+#		define strcasecmp _stricmp
+#		define strncasecmp _strnicmp
+#		define strdup _strdup
+#		define fdopen _fdopen
+#	endif
 #	ifndef mode_t
 #		define mode_t int
 #	endif
@@ -162,13 +165,16 @@ extern "C"
 #	endif
 #else
 	/* Includes for UNIX */
-#	if (defined(__linux__)) && (! defined(_LARGEFILE64_SOURCE)) && (! defined(NO_LARGEFILE64_SOURCE))
+#	if ((defined(__linux__)) && (! defined(_LARGEFILE64_SOURCE)) && (! defined(NO_LARGEFILE64_SOURCE)) && (! defined(_LARGEFILE_SOURCE)) && (! defined(_FILE_OFFSET_BITS)))
 		/* Define _LARGEFILE64_SOURCE or NO_LARGEFILE64_SOURCE to
 		 * quiet this warning.  Be sure to define this for ALL
 		 * code that uses LibNcFTP.
 		 */
 #		warning "<ncftp.h> is #defining _LARGEFILE64_SOURCE for you"
 #		define _LARGEFILE64_SOURCE 1
+#		define   Stat    stat64
+#		define   Lstat   lstat64
+#		define   Fstat   fstat64
 #	endif
 #	include <unistd.h>
 #	include <sys/types.h>
@@ -416,6 +422,17 @@ typedef struct FTPConnectionInfo {
 	char textEOLN[4];			/* Set automatically per platform. */
 	int asciiTranslationMode;		/* You may modify this. */
 	const char *manualOverrideFeatures;	/* You may modify this. */
+
+	/* Only set the following field if all the following are true:
+	 *   Your client FTP program is running on a private network (e.g. 192.168.1.0); and
+	 *   You are accessing a server outside your network (e.g., on the Internet); and
+	 *   Your network's firewall does NOT fix private addresses in PORT for you; and
+	 *   You know what your external IP address is.
+	 */
+	struct sockaddr_in clientKnownExternalAddr;	
+
+	int maxNumberOfSuccessivePASVAttempts;	/* You may modify this. */
+
 #if USE_SIO
 	char srlBuf[768];
 	SReadlineInfo ctrlSrl;			/* Do not use or modify. */
@@ -502,7 +519,7 @@ typedef struct MLstItem{
  * a user-installed signal handler.
  */
 #define kDefaultXferTimeout		600
-#define kDefaultConnTimeout		30
+#define kDefaultConnTimeout		10
 #define kDefaultCtrlTimeout		135
 #define kDefaultAbortTimeout		10
 #else
@@ -518,7 +535,7 @@ typedef struct MLstItem{
 
 /* Suggested timeout values, in seconds, if you use timeouts. */
 #define kSuggestedDefaultXferTimeout	600
-#define kSuggestedDefaultConnTimeout	30
+#define kSuggestedDefaultConnTimeout	10
 #define kSuggestedDefaultCtrlTimeout	135	/* 2*MSL, + slop */ 
 #define kSuggestedAbortTimeout		10
 
@@ -527,6 +544,8 @@ typedef struct MLstItem{
 
 #define kDefaultDataPortMode		kFallBackToSendPortMode
 
+#define kDefaultMaxNumberOfSuccessivePASVAttempts	3
+
 #define kRedialStatusDialing		0
 #define kRedialStatusSleeping		1
 
@@ -756,9 +775,9 @@ int WinStat64(const char *const path, struct WinStat64 *const stp);
 
 /* The following block may be changed by configure script */
 #ifndef Stat
-#define Stat stat64
-#define Lstat lstat64
-#define Fstat fstat64
+#define Stat stat64    /* set by configure at 2008-07-15 10:17:24 */
+#define Lstat lstat64    /* set by configure at 2008-07-15 10:17:24 */
+#define Fstat fstat64    /* set by configure at 2008-07-15 10:17:24 */
 #endif
 
 #define kFtwNoAutoGrowButContinue (-1)
@@ -946,6 +965,7 @@ char *GetPass(const char *const prompt, char *const pwbuf, const size_t pwbufsiz
 int FilenameExtensionIndicatesASCII(const char *const pathName, const char *const extnList);
 void StrRemoveTrailingSlashes(char *dst);
 int StrToBoolOrInt(const char *s);
+int FTPFixPrivateAddr(struct sockaddr_in *maybePrivateAddr, struct sockaddr_in *knownNonPrivateAddrToUseIfNeeded);
 #if defined(WIN32) || defined(_WINDOWS) || defined(__CYGWIN__)
 char *StrFindLocalPathDelim(const char *src);
 char *StrRFindLocalPathDelim(const char *src);
@@ -984,6 +1004,7 @@ int FTPSendPort(const FTPCIPtr cip, struct sockaddr_in *saddr);
 int FTPSendPassive(const FTPCIPtr cip, struct sockaddr_in *saddr, int *weird);
 int FTPSetStartOffset(const FTPCIPtr cip, longest_int restartPt);
 void FTPCloseControlConnection(const FTPCIPtr cip);
+int FTPSendCommandStr(const FTPCIPtr cip, char *const command, const size_t siz);
 int FTPSendCommand(const FTPCIPtr cip, const char *const cmdspec, va_list ap)
 #if (defined(__GNUC__)) && (__GNUC__ >= 2)
 __attribute__ ((format (printf, 2, 0)))
@@ -995,11 +1016,21 @@ void FTPManualOverrideFeatures(const FTPCIPtr cip);
 int FTPMListOneFile(const FTPCIPtr cip, const char *const file, const MLstItemPtr mlip);
 void FTPInitializeAnonPassword(const FTPLIPtr);
 void FTPInitIOTimer(const FTPCIPtr);
-int FTPStartDataCmd(const FTPCIPtr, int, int, longest_int, const char *,...)
+int FTPStartDataCmd(const FTPCIPtr, const int, const int, const longest_int, const char *const ,...)
 #if (defined(__GNUC__)) && (__GNUC__ >= 2)
 __attribute__ ((format (printf, 5, 6)))
 #endif
 ;
+int FTPStartDataCmd2(const FTPCIPtr cip, const int netMode, const int type, const longest_int startPoint, char *const cmdstr, const size_t cmdstrSize, const char *variableCommandSpec, ...)
+#if (defined(__GNUC__)) && (__GNUC__ >= 2)
+__attribute__ ((format (printf, 7, 8)))
+#endif
+;
+int FTPStartDataCmd3(const FTPCIPtr cip, const int netMode, const int type, const longest_int startPoint, char *const cmdstr, const size_t cmdstrSize, const char *const variableCommandSpec, va_list ap)
+#if (defined(__GNUC__)) && (__GNUC__ >= 2)
+__attribute__ ((format (printf, 7, 0)))
+#endif
+;
 void FTPStartIOTimer(const FTPCIPtr);
 void FTPStopIOTimer(const FTPCIPtr);
 void FTPUpdateIOTimer(const FTPCIPtr);
